% //////////////////////////////////////////////////////////
% //
% // synthetic test scenes for
% // structured light survey
% // 
% // christoph schmalz
% // 2009
% //
% // christoph(a)structuredlightsurvey.de
% //
% //////////////////////////////////////////////////////////
% //
% // version 1.0 - first public release
% // version 1.1 - better interpolation
% //
% //////////////////////////////////////////////////////////
% //
% //    loads measured and ground truth depth data
% //    interpolates ground truth z at measured x,y coords
% //    displays difference graphically and textually
% //
% //    differences larger than maxDeviation are outliers
% //    the colormap goes from -colorMaxDev to +colorMaxDev
% //
% //    needs nansuite and tinterp
% //////////////////////////////////////////////////////////

function result = compareDepthMapsXYZ( dataFile, groundTruthFile, ...
                                thinning, maxDeviation, ...
                                maxTriangleArea, maxTriangleEdgeLength, ...
                                colorMaxDev, verbose )

% todo: determine outlier cutoff automatically from sigma!                            

fileinfo = dir(dataFile);
if (fileinfo.bytes==0)
    result.outliers = 0;
    result.inliers = 0;
    result.errorMean = NaN;
    result.errorSigma = NaN;
    result.meanAbsError = NaN;
    return;
end

data = importdata(dataFile);
data = data(1:thinning:end,:);

groundTruth = importdata(groundTruthFile);
groundTruth = groundTruth(1:thinning:end,:);

%ground truth data is much denser
%so sample it at the locations where we have measured data

xm = data(:,1);
ym = data(:,2);
zm = data(:,3);
% xm = data(1:1000,1);
% ym = data(1:1000,2);
% zm = data(1:1000,3);

% Triangulate
t = delaunayn( groundTruth(:,1:2) );

% Interpolate
% (use triangulation as is, else strange errors may happen)
zi = tinterp(groundTruth(:,1:2),t,groundTruth(:,3),xm,ym,'quadratic');

% we have to remove the date generated by large triangles
% where interpolation is too risky
% (because we cannot remove the triangles themselves)
% additionally the triangulation returns the convex hull
% of the data. using this for interpolation creates false
% depth values 
% a) for background pixels when the measured object
%   is not convex
% b) for large jumps in z at object boundaries when 
%   x,y coords are noisy
badIndices = findLargeTriangles(t,groundTruth,maxTriangleArea,maxTriangleEdgeLength);

% find the enclosing triangle for each point
triIndex = tsearch(groundTruth(:,1),groundTruth(:,2),t,xm,ym);

% set the 'bad' points to NaN
pointsToIgnore = ismember( triIndex, badIndices );
zi( pointsToIgnore ) = NaN;

% calculate the deviation from ground truth for the remaining points
diff = zm-zi;

%check for outliers
result.outliers = sum (sum ( abs(diff)>maxDeviation ) );

%set outliers to NaN
diff ( abs(diff)>maxDeviation ) = NaN;

%calculate statistics
result.inliers = numel(diff) - sum(sum( isnan(diff) ));
result.errorMean = nanmean(nanmean(diff));
result.errorSigma = nanstd(reshape(diff,1,[]));

if verbose
    %the colormap interpolates between zero and colorMaxDev
%     color = min( abs(diff), colorMaxDev );
% 
%     figure;
%     scatter3( xm,ym,diff,3,color );
%     title('deviation from ground truth');
%     xlabel('x');
%     ylabel('y');
%     zlabel('z');
%     zlim( [-maxDeviation,maxDeviation] );
%     colormap cool;
%     colorbar;

    disp( sprintf('%d\t%d\t%d\t%d\n', ...
        result.outliers, result.inliers, ...
        result.errorMean, result.errorSigma ) );

end

%show triangulation - very slow
% figure;
% triplot(t, groundTruth(:,1),groundTruth(:,2));
% title('ground truth triangulation');
% xlabel('x');
% ylabel('y');


% %interpolate a smooth surface from the ground truth data
% figure;
% trisurf(t, groundTruth(:,1),groundTruth(:,2),groundTruth(:,3));
% lighting phong
% shading interp
% title('ground truth meshed data');
% xlabel('x');
% ylabel('y');
% zlabel('z');

% figure;
% scatter3( xm,ym,zm,'r.' );
% title('measured data');
% xlabel('x');
% ylabel('y');
% zlabel('z');
% 
% figure;
% scatter3( xm,ym,zi,'b.' );
% title('interpolated data');
% xlabel('x');
% ylabel('y');
% zlabel('z');
% 
% figure;
% scatter3( groundTruth(:,1),groundTruth(:,2),groundTruth(:,3),'g.' );
% title('ground truth data');
% xlabel('x');
% ylabel('y');
% zlabel('z');

function badIndices = findLargeTriangles( tri, verts, maxArea, maxEdgeLength )

bad = zeros(size(tri,1),1);

limit = maxEdgeLength^2;

for i=1:size(tri,1)
    %get vertices
    data = verts( tri(i,:), : );

    %calc area
    area = sqrt( sum( cross(data(2,:)-data(1,:),data(3,:)-data(1,:)).^2 ) )/2;
    if area>maxArea
        bad(i)=1;
        continue
    end
    
    %there are three edges to check
    e1 = data(2,:)-data(1,:);
    l1 = e1*e1';
    if l1>limit
        bad(i)=1;
        continue
    end

    e2 = data(3,:)-data(2,:);
    l2 = e2*e2';
    if l2>limit
        bad(i)=1;
        continue
    end

    e3 = data(1,:)-data(3,:);
    l3 = e3*e3';
    if l3>limit
        bad(i)=1;
        continue
    end
end

disp( sprintf('Ignoring %d large triangles, %d remaining!', sum(bad), size(tri,1)-sum(bad) ) );

badIndices = find(bad==1);