//////////////////////////////////////////////////////////
//
// synthetic test scenes for
// structured light survey
// 
// christoph schmalz
// 2009
//
// christoph(a)structuredlightsurvey.de
//
//////////////////////////////////////////////////////////
//
// version 1.0 - first public release
// version 1.1 - handle glass dragon automatically
// version 1.2 - stereo and more
//
//////////////////////////////////////////////////////////
//
// this is the master file that includes all others
//
// to use this you need a bitmap of the structured light
// pattern to project. save it as "pattern.png"
//
// the aleutian image can be obtained from
// www.environmentalgraffiti.com/wp-content/uploads/2008/06/aleutian_hires.jpg
// the heighmap is a grayscale version of the image
//
// the sun image can be obtained from
// http://www.solarphysics.kva.se/gallery/images/2004/basil_fleck.jpg
// the heighmap is a grayscale version of the image
//////////////////////////////////////////////////////////


//use alternative camera position for better overview
//also replace complex meshes by simple bounding boxes
//to eliminate parsing delay
#local TEST = 0;

//generate ground truth depthmap data
//do not forget to turn off antialiasing when using this
#local DEPTHMAP = 0; 

//camera position (0 mono perspective, 1 stereo left, 2 stereo right)
#local CAMPOS = 1;

//render with +KFF8 to generate all images in one go
#local OBJECT = frame_number;

//alternatively select each scene individually
#local OBJECT = 4; //plane test scene

//1 keyboard
//2 grid
//3 blade
//4 dragon
//5 ring
//6 cells
//7 aleut heightfield
//8 sun heightfield
//9 glass dragon

#include "testMaterials.inc"
#local currentMaterial = material0

//0 gray
//1 white
//2 almost black
//3 gold
//4 silver
//5 jade / glass volume scatterer (long rendering time!)
//6 stone

//special: object 9 is glass dragon
#if (OBJECT=9)
	#local OBJECT = 4;
	#local currentMaterial = material5;
#end

//set the pattern to use here
#local imageFile = "patternW6H2S4.png";

//--------------------------------------------------
//	global
//--------------------------------------------------
#local aspectRatio = image_width/image_height;

#if (DEPTHMAP)
	//do not mess with the gamma!
#else
	global_settings 
	{ 
		assumed_gamma 1.0	
	}
#end

//sky
sky_sphere {
pigment {
  gradient y
  color_map {
    [(1-cos(radians( 30)))/2 color <1,0,0>]
    [(1-cos(radians(120)))/2 color <0,0,1>]
  }
  scale 2
  translate -1
}
}

#if (TEST)
	//background
	plane { z, 1.5 hollow
		texture { pigment { checker rgb 0.2, rgb 0.8 scale 0.1 } }
	}
	//ground
	plane { y, -1.5 hollow
		texture { pigment { checker rgb <0.8,0.2,0.2>, rgb 0.8 scale 0.1 } }
	}
	
	#include "axes.inc"
	object
	{
		Axes( 0.1, <0,0,0> )
		no_shadow
	}
#else
	//background
	plane { z, 0.4 hollow //important! one half-space is "inside"
		texture { 
#if (DEPTHMAP)
			pigment { color rgb 0 } 
			finish { ambient 1 diffuse 0 }
#else
			#local backgroundFile = "white.png"

			pigment {
				/*
				image_map {
			     	png backgroundFile
			     	interpolate 2
			     	once
			    	}
			    scale 8
			    translate <-4,-4,0>
			    */
			    
			    color rgb 1 //white wall
				}
#end
			}
	}
#end

//--------------------------------------------------
//	the structured light system
//--------------------------------------------------

//#local extraLightColor = <0,0,0.2*(frame_number-1)>;
#local extraLightColor = 0.5;

#include "structuredLight.inc"
structuredLight( aspectRatio, imageFile, CAMPOS, extraLightColor, TEST )

//--------------------------------------------------
//	special depth-coding texture
//--------------------------------------------------
#if (DEPTHMAP)
	#include "depthCodeTexture.inc"
	//assume working space between 0.5 and 1.5 meters
	//important: transform the texture according to the
	//current camera position!
	#local currentMaterial = depthCodingTexture(0.5,1.5,currentCamPosition,currentCamDirection)
	//we have 256*100 steps for one unit
	//ie resolution is 40 microns
#end

//--------------------------------------------------
// objects
//--------------------------------------------------

#switch (OBJECT)

#case (1) //--------------------------------
	#include "keyboard.inc"
	#local color1 = 0.6;
	#local color2 = 0.1;
	object {
		#if (DEPTHMAP)
			keyboard(color1,color2,0)
		#else
			keyboard(color1,color2,1)
		#end
		scale 0.7
		rotate y*180
		rotate -z*90
		#if (DEPTHMAP)
			//note: material must be very last declaration
			//otherwise depth coding does not work as intended
			material { currentMaterial } 
		#end
	}

#break

#case (2) //--------------------------------
	#local gridSize = 100;
	#local gridPitch = 1/(2*gridSize-1);
	#local rod = box { 0, 1 scale <gridPitch*0.8,gridPitch*0.8,1> } //rod in z-direction
	union {	
		#local index = 0;
		#while (index<gridSize)
			object { rod translate <index*2*gridPitch,0,0> }
			object { rod rotate y*90 translate <0,0,index*2*gridPitch+gridPitch> }
			#declare index = index+1;
		#end
		rotate -x*90
		translate -x*0.5
		translate -y*0.5
		scale <2,2,3>
		rotate z*40
		rotate y*10
		hollow
		material { currentMaterial }
	}

#break

#case (3) //--------------------------------
	#include "turbineBlade.inc"
	object {
		turbineBlade(TEST)
		scale 0.7
		rotate z*90
		rotate x*180
		rotate -y*20
		material { currentMaterial }
	}
#break

#case (4) //--------------------------------
	#include "dragon.inc"
	object {
		dragon(TEST)
		scale 0.7
		material { currentMaterial }
	}
#break

#case (5) //--------------------------------
	/*
	//crackled surface
	//parameters see http://www.imagico.de/pov/cr_form.html
	//and http://www.imagico.de/pov/cr_metric.html
	#local cracklefunc = function {pattern {crackle form <0.17,0.96,0> }}

	isosurface {
		function { y - cracklefunc(x*50,0,z*50) * 0.03 }

	    max_gradient 20
	    contained_by{box{<-1,-0.01,-1><1,0.3,1>}}
	    
	    pigment { crackle form <0.17,0.96,0> scale 0.02 }

	    rotate < -90,0,0 >
	    translate z*0.15
	}
	*/
		
	#local P=function{
		pigment{
			leopard
			colour_map{	[0.0  rgb 0.3]
						[0.1  rgb 0.0]
						[1.0  rgb 1.0]}
			scale 0.001
			}}
	
	isosurface {
	  function { y - P(x,0,z).gray*0.01 }
	        max_gradient 10
	        contained_by{box{<-1,-0.1,-1>,<1,0.1,1>}}
	        rotate -x*90
	        translate z*0.15
	        
	        material { currentMaterial }
	}

	#include "shapes.inc"
	
	object {
		Supertorus(
		  1.00, 0.1,// R_Major, R_Minor,
		  1.00, 0.45,// Maj_Control, Min_Control,
		  0.001,1.50)// Accuracy, Max_Gradient)
		  
		scale 0.1
		rotate -x*90
		translate -z*0.15
	    #if (DEPTHMAP)
			material { currentMaterial }
		#else
			material { material4 }
		#end
		}
	
#break

#case (6) //--------------------------------
	// heightfield with cells pigment
	height_field { 
	    function 2000,2000 { //samples
		pigment {
		    cells 
		    color_map {
		       [0 rgb 0]
		       [1 rgb 1]
		    }
		    scale .006
		}
	    }
	    translate -0.5
	    rotate x*90 
	    scale <2,2,0.01> 
		material { currentMaterial }
	}

#break

#case (7) //--------------------------------

	#declare aleutianMaterial =
    material {
    	texture {
		    pigment { 
		    	image_map
		   		{ png "aleutianTexture.png"
		   			interpolate 2
		   			}
		   	}
		}
	    translate <-.5, -.5, -.5>
	    scale <2, 0.1, 2>
	    rotate -x*90
	}


	//height map
  	height_field {
	    png "aleutianHeightmap.png"
	    smooth
	    translate <-.5, -.5, -.5>
	    scale <2, 0.1, 2>
	    rotate -x*90
	
	    #if (DEPTHMAP)
		    material { currentMaterial }
		#else
		    material { aleutianMaterial }
		#end
	}

#break

#case (8) //--------------------------------

	#declare sunMaterial =
    material {
    	texture {
		    pigment { 
		    	image_map
		   		{ png "sunTexture.png"
		   			interpolate 2
		   			}
		   	}
		}
	    translate <-.5, -.5, -.5>
	    scale <2, 0.1, 2>
	    rotate -x*90
	}

	//height map
	height_field {
	    png "sunHeightmap.png"
	    smooth
	    translate <-.5, -.5, -.5>
	    scale <2, 0.1, 2>
	    rotate -x*90
	    
	    #if (DEPTHMAP)
		    material { currentMaterial }
		#else
		    material { sunMaterial }
		#end
	}

#break

//case (9) is glass dragon, see above!

#case (10) //--------------------------------
	
	union
	{
	
		plane { z, 0
				hollow
				material { currentMaterial } }
				
//		cylinder { <0,-1,0> <0,1,0> 0.01
//					material { currentMaterial }
//					}

		//make it so that the projector is orthogonal					
		//rotate <0,degrees(atan(-0.25)),0>

		translate z*0
	}
#break

#end
